==> s-e-1.0.3.tgz: package/src/generate/feature/index.js <==
const fs = require('fs');
const path = require('path');
const {pluralize, titleCase, removeTmpl} = require('../../utils/index');

const settings = {
    path: {
        features: `.${path.sep}src${path.sep}feature${path.sep}`,
        schema: `.${path.sep}schema.graphql`,
        routes: `.${path.sep}Routes.js`
    },
    tmpl: {
        path: './src/generate/feature/templates',
        folders: [
            'api', 
            'form',
            'i18n',
            'layout',
            `api${path.sep}graphql`,
        ],
        files: [
            `api${path.sep}graphql${path.sep}Get{features}.graphql.tmpl`,
            // // `api${path.sep}graphql${path.sep}Get{feature}ByPk.graphql`,
            // // `api${path.sep}graphql${path.sep}Create{feature}.graphql`,
            // // `api${path.sep}graphql${path.sep}Update{feature}ByPk.graphql`,
            // // `api${path.sep}graphql${path.sep}Delete{feature}ByPk.graphql`,
            `api${path.sep}use{features}.js.tmpl`,
            // // `api${path.sep}use{feature}.js`,
            // // `form${path.sep}Edit{feature}Form.js`,
            `i18n${path.sep}en.json.tmpl`,
            `i18n${path.sep}lt.json.tmpl`,
            // // `layout${path.sep}Edit{feature}Layout.js`,
            `layout${path.sep}{features}ListLayout.js.tmpl`,
            `routes.js.tmpl`
        ]
    }
}

const generateFolder = name => {
    return new Promise((resolve) => {
        fs.mkdir(name, (err) => {
            if (err) throw err;
            resolve('ok')
        })
    })
}

// generate feature strcuture
const generateFolders = async (name) => {
    const mainFolder = settings.path.features + name;
    await fs.mkdir(mainFolder, async (e) => {
        if (e) throw e;
        const generation = await Promise.all(settings.tmpl.folders.map(f => generateFolder(mainFolder + path.sep + f)));
        console.info('1/3 - folders generated successfully!');
    })
}


// generate file
const generateFile = (name, content) => {
    return new Promise((resolve) => {
        fs.writeFile(name, content, (err) => {
            if (err) throw err;
            resolve('ok');
        });
    })
} 

// read tmpl
const readTmpl = (tmpl) => {
    return new Promise((resolve) => {
        fs.readFile(tmpl, 'utf-8', async (err, data) => {
            const fileName = removeTmpl(tmpl.replace('{features}', titleCase(plural)));
            const content = data
                .replace(/{{feature}}/g, name)
                .replace(/{{features}}/g, titleCase(plural))
                .replace(/{{table}}/g, table);

            await generateFile(mainFolder + path.sep + fileName, content);
        });
    });
}

// generate files from templates
const generateFiles = async (name, table) => {
    const mainFolder = settings.path.features + name;
    const fields = ['id', 'cid', 'name'];   // TODO: introspect from schema.graphql
    const plural = pluralize(name);

    const generation = await Promise.all(settings.tmpl.files.map(tmpl => readTmpl(settings.tmpl.path + path.sep + tmpl)));
    console.info('2/3 - files generated successfully!');
}


const generateFeature = async ({name, table}) => {
    await generateFolders(name);
    await generateFiles(name, table);

    // add to Routes.js
}

module.exports = {
    generateFeature,
}

==> s-e-1.0.4.tgz: package/src/generate/feature/index.js <==
const fs = require('fs');
const path = require('path');
const {pluralize, titleCase, removeTmpl} = require('../../utils/index');

const settings = {
    path: {
        features: `.${path.sep}src${path.sep}feature${path.sep}`,
        schema: `.${path.sep}schema.graphql`,
        routes: `.${path.sep}Routes.js`
    },
    tmpl: {
        path: './src/generate/feature/templates',
        folders: [
            'api', 
            'form',
            'i18n',
            'layout',
            `api${path.sep}graphql`,
        ],
        files: [
            `api${path.sep}graphql${path.sep}Get{features}.graphql.tmpl`,
            // // `api${path.sep}graphql${path.sep}Get{feature}ByPk.graphql`,
            // // `api${path.sep}graphql${path.sep}Create{feature}.graphql`,
            // // `api${path.sep}graphql${path.sep}Update{feature}ByPk.graphql`,
            // // `api${path.sep}graphql${path.sep}Delete{feature}ByPk.graphql`,
            `api${path.sep}use{features}.js.tmpl`,
            // // `api${path.sep}use{feature}.js`,
            // // `form${path.sep}Edit{feature}Form.js`,
            `i18n${path.sep}en.json.tmpl`,
            `i18n${path.sep}lt.json.tmpl`,
            // // `layout${path.sep}Edit{feature}Layout.js`,
            `layout${path.sep}{features}ListLayout.js.tmpl`,
            `routes.js.tmpl`
        ]
    }
}

const generateFolder = name => {
    return new Promise((resolve) => {
        fs.mkdir(name, (err) => {
            if (err) throw err;
            resolve('ok')
        })
    })
}

// generate feature strcuture
const generateFolders = async (name) => {
    const mainFolder = settings.path.features + name;
    await fs.mkdir(mainFolder, async (e) => {
        if (e) throw e;
        const generation = await Promise.all(settings.tmpl.folders.map(f => generateFolder(mainFolder + path.sep + f)));
        console.info('1/3 - folders generated successfully!');
    })
}


// generate file
const generateFile = (file, content) => {
    return new Promise((resolve) => {
        fs.writeFile(file, content, (err) => {
            if (err) throw err;
            resolve('ok');
        });
    })
} 

// read tmpl
const readTmpl = (name, table, tmpl) => {
    const mainFolder = settings.path.features + name;
    const fields = ['id', 'cid', 'name'];   // TODO: introspect from schema.graphql
    const plural = pluralize(name);

    return new Promise((resolve) => {
        fs.readFile(settings.tmpl.path + path.sep + tmpl, 'utf-8', async (err, data) => {
            const fileName = removeTmpl(tmpl.replace('{features}', titleCase(plural)));
            const content = data
                .replace(/{{feature}}/g, name)
                .replace(/{{features}}/g, titleCase(plural))
                .replace(/{{table}}/g, table);

            await generateFile(mainFolder + path.sep + fileName, content);

            resolve('ok');
        });
    });
}

// generate files from templates
const generateFiles = async (name, table) => {
    const generation = await Promise.all(settings.tmpl.files.map(tmpl => readTmpl(name, table, tmpl)));
    console.info('2/3 - files generated successfully!');
}


const generateFeature = async ({name, table}) => {
    await generateFolders(name);
    await generateFiles(name, table);

    // add to Routes.js
}

module.exports = {
    generateFeature,
}
